@@ -15,19 +15,14 @@ from selenium.webdriver.chrome.service import Service
 from selenium.webdriver.common.by import By
 from selenium.webdriver.support.ui import WebDriverWait
 from selenium.webdriver.support import expected_conditions as EC
-from retrying import retry
-
-
-def kill_all_by_names(process_names):
-    processes = psutil.process_iter(['pid', 'name'])
-    for process_name in process_names:
-        for process in processes:
-            try:
-                if process_name in process.info['name'] and process.is_running():
-                    process.terminate()
-                    process.wait(timeout=3)
-            except Exception as ex:
-                print(ex)
+from .utils import terminate_processes_by_names
+from tenacity import (
+    retry,
+    retry_if_result,
+    stop_after_attempt,
+    wait_fixed
+)
+import subprocess
 
 
 def encrypt_sha1(fpath: str) -> str:
@@ -52,19 +47,20 @@ def download_file(url, save_path):
 
 class SuperBrowserAPI:
 
-    def __init__(self, company, username, password, _print=print):
+    def __init__(self, company, username, password, _print=print, xbot=None):
+        self.current_cef = "superbrowser"
+        self.main_browser_running = False
         self.exe_path = self.get_super_browser_exe_path()
         self.print = _print
         # 获取当前运行路径
         self.driver_folder_path = os.path.join(os.getcwd(), "ziniao_driver")
         self.download_driver(self.driver_folder_path)
-
         assert company and username and password, "登录信息都是必选,请检查"
         self.user_info = {"company": company, "username": username, "password": password}
         self.socket_port = None
         # 初始化一个端口
-        self.socket_port = self.get_port()
         self.web_driver = None
+        self.xbot = xbot
 
     def get_driver(self, open_ret_json):
         core_type = open_ret_json.get('core_type')
@@ -79,6 +75,9 @@ class SuperBrowserAPI:
         else:
             return None
 
+    def get_cef(self):
+        return self.current_cef
+
     def download_driver(self, driver_folder_path):
         config_url = "https://cdn-superbrowser-attachment.ziniao.com/webdriver/exe_32/config.json"
         response = requests.get(config_url)
@@ -112,10 +111,8 @@ class SuperBrowserAPI:
                 self.print(f"驱动{filename}不存在，开始下载...")
                 download_file(item['url'], local_file_path)
 
-    @staticmethod
-    def get_super_browser_exe_path():
+    def get_super_browser_exe_path(self):
         """获取紫鸟浏览器启动文件路径"""
-
         config_path = os.path.join(os.path.expanduser('~'), 'AppData', 'Local', 'ShadowBot', 'ChromiumBrowser.config')
 
         if not os.path.exists(config_path):
@@ -124,17 +121,15 @@ class SuperBrowserAPI:
         # 解析XML文件
         tree = ET.parse(config_path)
         root = tree.getroot()
-
-        # 查找匹配的产品名称
-        matching_nodes = root.findall(".//ChromiumBrowserInfo[ProductName='{}']".format("superbrowser"))
+        # 提取ProcessName和ExePath
+        matching_nodes = root.findall(".//ChromiumBrowserInfo")
         if len(matching_nodes) == 0:
             raise Exception("未安装紫鸟浏览器插件，请在影刀中安装插件")
-        # 提取ProcessName和ExePath
         node = matching_nodes[0]
-        ProcessName, ExePath = node.find('ProcessName').text, node.find('ExePath').text
-        assert ProcessName == "superbrowser", "紫鸟浏览器插件未正确安装，请检查"
-        assert os.path.basename(ExePath) != "superbrowser.exe", "紫鸟浏览器插件未正确安装，请检查"
-        return ExePath
+        product_name, process_name, exe_path = node.find('ProductName').text, node.find('ProcessName').text, node.find('ExePath').text
+        assert product_name in ["superbrowser", "ziniaobrowser"] and process_name in ["superbrowser", "ziniaobrowser"], "紫鸟浏览器插件未正确安装，请检查"
+        self.current_cef = process_name
+        return exe_path
 
     def get_port(self):
         procarr = []
@@ -156,29 +151,32 @@ class SuperBrowserAPI:
         启动紫鸟客户端
         :return:
         """
-        self.kill_all_super_browser()
-        self.kill_all_store_process()
-        cmd_text = None
-        try:
-            self.socket_port = self.get_port()
-            cmd_text = [self.exe_path, '--run_type=web_driver', '--ipc_type=http', '--port=' + str(self.socket_port)]
-            self.print(" ".join(cmd_text))
-            subprocess.Popen(cmd_text)
-            self.print("start ..")
-            time.sleep(3)
-        except Exception as e:
-            self.print("start_ExeBrowser err...", e)
+        if self.main_browser_running:
+            self.kill_all_super_browser()
+            self.kill_all_store_process()
+        # 尝试启动3次
+        try_max_count = 0
+        while try_max_count < 3 and not self.main_browser_running:
             try:
                 self.socket_port = self.get_port()
+                cmd_text = [self.exe_path, '--run_type=web_driver', '--ipc_type=http', '--port=' + str(self.socket_port)]
+                self.print(" ".join(cmd_text))
                 subprocess.Popen(cmd_text)
+                self.print("start ..")
                 time.sleep(3)
+                # 更新内核
+                self.update_core()
+                # 如果执行没报错就是启动成功
+                self.main_browser_running = True
             except Exception as e:
-                self.print('start browser process failed', e)
-                raise Exception(f'start browser process failed {e}')
+                self.print("start_ExeBrowser err...", e)
+            finally:
+                try_max_count += 1
+        return self.main_browser_running
 
     @retry(
-        stop_max_attempt_number=10,  # 最大重试次数
-        wait_fixed=1000,  # 每次重试间隔 1 秒
+        stop=stop_after_attempt(10),  # 最大重试次数
+        wait=wait_fixed(1),  # 每次重试间隔10秒
     )
     def send_http(self, data):
         """
@@ -187,7 +185,6 @@ class SuperBrowserAPI:
         :return:
         """
         try:
-            sleep(1)
             url = 'http://127.0.0.1:{}'.format(self.socket_port)
             # response = requests.post(url, json.dumps(data).encode('utf-8'), timeout=120)
             response = requests.post(url, json=data, timeout=60)
@@ -200,7 +197,47 @@ class SuperBrowserAPI:
             else:
                 raise Exception(json.dumps(r, ensure_ascii=False))
         except Exception as err:
-            raise
+            raise err
+
+    def update_core(self):
+        """
+        下载所有内核，打开店铺前调用，需客户端版本5.285.7以上
+        因为http有超时时间，所以这个action适合循环调用，直到返回成功
+        """
+        data = {
+            "action": "updateCore",
+            "requestId": str(uuid.uuid4()),
+        }
+        data.update(self.user_info)
+        while True:
+            result = self.send_http(data)
+            self.print(result)
+            if result is None:
+                self.print("等待客户端启动...")
+                time.sleep(2)
+                continue
+            if result.get("statusCode") is None or result.get("statusCode") == -10003:
+                self.print("当前版本不支持此接口，请升级客户端")
+                return
+            elif result.get("statusCode") == 0:
+                self.print("更新内核完成")
+                return
+            else:
+                self.print(f"等待更新内核: {json.dumps(result)}")
+                time.sleep(2)
+
+    def get_running_info(self):
+        """
+        下载所有内核，打开店铺前调用，需客户端版本5.285.7以上
+        因为http有超时时间，所以这个action适合循环调用，直到返回成功
+        """
+        data = {
+            "action": "getRunningInfo",
+            "requestId": str(uuid.uuid4()),
+        }
+        data.update(self.user_info)
+        result = self.send_http(data)
+        return result.get("browsers")
 
     def open_store(self, store_info,
                    close_other_store=True,
@@ -217,6 +254,15 @@ class SuperBrowserAPI:
         """
         打开店铺
         """
+        # 控制下载目录
+        download_shop_path = os.path.join(os.path.expanduser('~'), 'Desktop', 'Super Browser', store_info)
+        # 如果目录存在则删除（包括所有内容）
+        if os.path.exists(download_shop_path):
+            shutil.rmtree(download_shop_path)  # 递归删除整个目录
+        # 创建目录（包括所有必要的中间目录）
+        os.makedirs(download_shop_path, exist_ok=True)
+        self.print(f"目录 {download_shop_path} 已成功重置")
+
         requestId = str(uuid.uuid4())
         data = {
             "action": "startBrowser",
@@ -227,12 +273,13 @@ class SuperBrowserAPI:
             "isWebDriverReadOnlyMode": isWebDriverReadOnlyMode,
             "cookieTypeLoad": cookieTypeLoad,
             "cookieTypeSave": cookieTypeSave,
-            "runMode": "2",
+            "runMode": "3",
             "isLoadUserPlugin": False,
             "notPromptForDownload": 0,
             "pluginIdType": 1,
             "pluginIdList": pluginIdList,
             "privacyMode": isprivacy,
+            "forceDownloadPath": download_shop_path,
         }
         data.update(self.user_info)
 
@@ -243,14 +290,35 @@ class SuperBrowserAPI:
         res_result = self.send_http(data)
 
         self.web_driver = self.get_driver(res_result)
-        # 隐式等待 30s
+        # 隐式等待 5s
         self.web_driver.implicitly_wait(5)
 
-        # 检测ip
-        if self.open_ip_check("https://4.ipw.cn/", res_result.get('ipDetectionPage'), res_result):
-            return res_result
-        else:
-            raise Exception("ip检测失败")
+        # 优化IP
+        self.optimization_ip(res_result.get('ipDetectionPage'))
+        # 创建并返回
+        return res_result, self.get_web_page("https://www.baidu.com/", timeout=60) if self.xbot else None
+
+    def get_web_page(self, url, timeout=60):
+        """
+        进入主页，如果插件加载失败则重试
+        """
+        start_time = time.time()
+        while True:
+            try:
+                web_page = self.xbot.web.create(url, mode=self.get_cef(), load_timeout=0)
+                try:
+                    web_page.wait_load_completed(10)
+                except:
+                    pass
+                return web_page
+            except Exception as e:
+                if '等待页面加载超时' in str(e):
+                    web_page = self.xbot.web.get(url=f"{url}", mode=self.get_cef(), load_timeout=0, use_wildcard=True)
+                    return web_page
+                if time.time() - start_time < timeout:
+                    sleep(2)
+                else:
+                    raise e
 
     def find_element_by_xpath(self, xpath):
         try:
@@ -259,12 +327,10 @@ class SuperBrowserAPI:
             self.print(f"找不到元素 {xpath}")
             return None
 
-    def open_ip_check(self, ip_check_url, ip_detection_page: str, res_result: dict):
+    def optimization_ip(self, ip_detection_page: str):
         """
         打开ip检测页检测ip是否正常
         :param ip_detection_page:
-        :param res_result:
-        :param ip_check_url ip检测页地址
         :return 检测结果
         """
         # 先进入紫鸟官方插件,可自动优化IP
@@ -273,27 +339,12 @@ class SuperBrowserAPI:
         if next_button:
             next_button.click()
         try:
-            next_button = WebDriverWait(self.web_driver, 30).until(
+            next_button = WebDriverWait(self.web_driver, 60).until(
                 EC.element_to_be_clickable((By.XPATH, '//span[text()="打开账号"]/..'))
             )
         except Exception as ex:
             self.print("等待打开店铺按钮异常:" + traceback.format_exc())
-        current_url = self.web_driver.current_url
-        max_retry = 5
-        check_flag = False
-        while current_url != ip_check_url and max_retry > 0 and not check_flag:
-            try:
-                self.web_driver.get(ip_check_url)
-                current_url = self.web_driver.current_url
-                web_ip = self.web_driver.find_element(By.XPATH, '//body').text
-                if web_ip and str(web_ip).strip() == res_result.get("ip"):
-                    check_flag = True
-            except Exception as e:
-                self.print("ip检测异常:" + traceback.format_exc())
-            finally:
-                time.sleep(1)
-                max_retry -= 1
-        return check_flag
+        self.print("检测IP完毕")
 
     def close_store(self, ziniao_shop_id):
         request_id = str(uuid.uuid4())
@@ -346,13 +397,20 @@ class SuperBrowserAPI:
             except Exception as ex:
                 pass
 
-    @staticmethod
-    def kill_all_store_process():
-        kill_all_by_names(["superbrowser", "chromedriver"])
+    def kill_all_store_process(self):
+        # 先执行正常关闭,保留cookie,需要try catch防止没执行后面的过程
+        if self.main_browser_running:
+            try:
+                for browser_info in self.get_running_info():
+                    self.close_store(browser_info.get("browserOauth"))
+            except:
+                pass
+        terminate_processes_by_names(["superbrowser", "chromedriver", "ziniaobrowser"])
 
-    @staticmethod
-    def kill_all_super_browser():
-        kill_all_by_names(["SuperBrowser", "ziniao", "chromedriver"])
+    def kill_all_super_browser(self):
+        if self.main_browser_running:
+            self.get_exit()
+        terminate_processes_by_names(["SuperBrowser", "ziniao", "chromedriver"])
 
     def get_exit(self):
         """
@@ -360,8 +418,9 @@ class SuperBrowserAPI:
         :return:
         """
         data = {"action": "exit", "requestId": str(uuid.uuid4())}
-        # data.update(self.user_info)
-        self.print('@@ get_exit...' + json.dumps(data))
+        data.update(self.user_info)
+        self.print('@@ get_exit...' + json.dumps(self.send_http(data)))
+        self.main_browser_running = False
         self.kill_all_store_process()
         self.kill_all_super_browser()
 
